import 'package:chopper/chopper.dart';
import 'package:adeptarch/src/api/backend_api.dart' as BackendAPI;
import 'package:adeptutils/src/extendables/jsonable.dart';

/**
 * Created by krrigan on 02/27/19.
 */

abstract class APIService<I extends ChopperService, DataType extends Jsonable> implements ChopperService
{
    I service;
    DataType dataType;

    APIService(I service, {this.dataType, BackendAPI.BackendAPI backendAPI})
    {
      this.service = service;
      if(backendAPI != null) backendAPI.addService(service); else
          BackendAPI.backendAPI.addService(service);
    }

    dynamic getAuthorization();

    Future<Response> getSaveResponse(Future<Response> futureResponse) async{
      if(futureResponse == null) return null;
      Response response;
      try{
          response = await futureResponse;
      } catch(e){
          if(e is Response) response = e; else throw e;
      }
      return response;
    }

    Future<Response<DataType>> parseResponse(Future<Response> futureResponse) async{
      return genericParseResponse(futureResponse, dataType);
    }

    Future<Response<List<DataType>>> parseResponseAsList(Future<Response> futureResponse) async{
      return genericParseResponseAsList(futureResponse, dataType);
    }

    Future<Response<DataType>> genericParseResponse<DataType extends Jsonable>(Future<Response> futureResponse, DataType dataType) async{
      Response response = await getSaveResponse(futureResponse);
      try{
        if(dataType != null){
          DataType dataTypeResult = response.body is String ? dataType.fromJsonString(response.body) : dataType.fromJsonMap(response.body);
          return Response<DataType>(response.base, dataTypeResult);
        }
      }catch(e){
        String message = e.toString();
        print(e);
      }
      return Response<DataType>(response.base, null);
    }

    Future<Response<List<DataType>>> genericParseResponseAsList<DataType extends Jsonable>(Future<Response> futureResponse, DataType dataType) async{
      Response response = await getSaveResponse(futureResponse);
      try{
        if(dataType != null){
          List<DataType> dataList = response.body is String ? dataType.fromJsonStringList(response.body) : dataType.fromJsonList(response.body);
          return Response<List<DataType>>(response.base, dataList);
        }
      }catch(e){
        String message = e.toString();
        print(e);
      }
      return Response<List<DataType>>(response.base, null);
    }

    @override
    void dispose() => service.dispose();

    @override
    Type get definitionType => service.definitionType;

    @override
    ChopperClient get client => service.client;

    @override
    set client(ChopperClient client) => service.client = client;
}
